; --==ATOM==--
; Advanced Tile Object Manipulator
;
; by David Phillips <david@acz.org>
; Assembly Coder's Zenith
; http://www.acz.org/
;

#include "ti86asm.inc"

;=========================================================================
; Startup Code
;=========================================================================
 .org _asm_exec_ram
 
 nop
 jp ProgStart
 .dw 0
 .dw ShellTitle

;=========================================================================
; Memory locations
;=========================================================================
VideoRam	= $f800			; [1024]  \   Must be
VideoRam2	= $fc00			; [1024]  /  contiguous

GrayMem1	= $8000			; [1024]  \   Must be
GrayMem2	= $8400			; [1024]  /  contiguous

Buffer1		= $8800			; [1024]  \   Must be
Buffer2		= $8c00			; [1024]  /  contiguous

AVars		= $9000			; [96]
Map 		= $9060			; [128]
Anims		= $90e0			; [120]   20 x (frames[1], time[1], spriteptr[2], coords[2])
Temp		= $9158			; [168]
Vars		= $9200			; [256]
ZeldaDat	= $9300			; [...]

Anim		= AVars			; [1]
MapI		= AVars + 1		; [20]
CurrTile	= AVars + 21	; [1]
NumMaps		= AVars + 22	; [1]

MapIUp		= MapI			; [1]
MapIDown	= MapI + 1		; [1]
MapILeft	= MapI + 2		; [1]
MapIRight	= MapI + 3		; [1]
MapITel1	= MapI + 4		; [1]
MapITel1S	= MapI + 5		; [1]
MapITel1Y	= MapI + 6		; [1]
MapITel1X	= MapI + 7		; [1]
MapITel2	= MapI + 8		; [1]
MapITel2S	= MapI + 9		; [1]
MapITel2Y	= MapI + 10		; [1]
MapITel2X	= MapI + 11		; [1]
MapITel3	= MapI + 12		; [1]
MapITel3S	= MapI + 13		; [1]
MapITel3Y	= MapI + 14		; [1]
MapITel3X	= MapI + 15		; [1]
MapITel4	= MapI + 16		; [1]
MapITel4S	= MapI + 17		; [1]
MapITel4Y	= MapI + 18		; [1]
MapITel4X	= MapI + 19		; [1]

;=========================================================================
; Masked sprite routines
#include "sprite.asm"
;=========================================================================
; * PutSprite		(not used)
; * PutSpriteClip	(not used)
; * GraySprite
; * GraySpriteClip

;=========================================================================
; RLE extractor
#include "rle.asm"
;=========================================================================
;  * DispRLE

;=========================================================================
; Grayscale library
#include "gray.asm"
;=========================================================================
;  * OpenGray
;  * CloseGray
;  * ClearGray

;=========================================================================
; Tripple buffering routines
#include "buffers.asm"
;=========================================================================
;  * ShowGray
;  * CopyBuffers
;  * ClearBuffers

;=========================================================================
; Scrolling routines
#include "scroll.asm"
;=========================================================================
;  * GrayScrollUp
;  * GrayScrollDown
;  * GrayScrollLeft
;  * GrayScrollRight
;  * GrayWarpMap
;  * ContrastUp
;  * ContrastDown

;=========================================================================
; Movement routines
#include "movement.asm"
;=========================================================================
;  * TryMoveUp
;  * TryMoveDown
;  * TryMoveLeft
;  * TryMoveRight
;  * CheckSpriteVert
;  * CheckSpriteHorz
;  * GetTile
;  * GetTileNum
;  * GetTileNumCenter
;  * GetTileCent
;  * CenterCoords
;  * GetTileHL
;  * CheckTile
;  * GetCoords
;  * UpdateAnim
;  * CheckCollision
;  * CheckPoint
;  * GrayLoadDrawSprite
;  * GrayDrawSprite

;=========================================================================
; Tile Object routines
#include "tiles.asm"
;=========================================================================
;  * LoadMapNum
;  * CheckWarpUp
;  * CheckWarpDown
;  * CheckWarpLeft
;  * CheckWarpRight
;  * LoadMapInfo
;  * CheckTile
;  * EnterNewTile
;  * GrayDrawMap
;  * GrayDrawTileMap

;=========================================================================
; Animation routines
#include "anim.asm"
;=========================================================================
;  * ClearAnims
;  * UpdateAnims
;  * AddAnim


.end
__________________________________________________________________
Copyright (C) 1999 - Assembly Coder's Zenith - http://www.acz.org/
