/*****************************************************************\
|                   Assembly Coder's Zenith                       |
|                     http://www.acz.org/                         |
|                                                                 |
|              ZEdit v0.15 -- editor for Zelda 86                 |
|                                                                 |
| David Phillips <david@acz.org>                                  |
|                                                                 |
| Inital     : 07/26/98 -- DP                                     |
| Started    : 01/31/99 -- DP                                     |
| Last Update: 05/11/99 -- DP                                     |
|                                                                 |
| Permission is granted to use this code in your own programs.    |
| If you would like to improve on this editor or use it as a base |
| to write your own editor, you are free to do so.  Please do     |
| not release a modified version without our permission, however. |
|                                                                 |
| This should compile under DOS 16 or 32 bit compilers.           |
| You will have to change the graphics primitives to your own     |
| systems as well.  It was designed to be used with 256 color     |
| modes, but could work with any modes without much modification. |
\*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include "ti86x.h"

//======== TYPEDEFS/STRUCTS ===============================================

typedef struct
{
  char edges[4],
       num_warps,
       warps[4][4],
       num_people,
       people[4][2],
       num_enemies,
       enemies[10][3];
} MapInfo;

typedef struct
{
  char    num_maps,
          screens[256][128],
          chests[100][3];
  MapInfo info[256];
} Map;

//======== GLOBAL VARIABLES ===============================================

char numspr;

char sprites[256][16];     // (8 x 8) 16-byte grayscale sprites 
char screen[16][8];        // working 16 x 8 screen (128 bytes)
Map  map;

//======== GRAPHICS PRIMITIVES ============================================
#ifdef __WATCOMC__
  #include <graph.h>
  void set_gfx_mode() 
  {
    _setvideomode(_MRES256COLOR);
  }
  void set_text_mode()       
  {
    _setvideomode(_TEXTC80);
  }
  void put_pixel(short x, short y, char c)
  {
    _setcolor(c);
    _setpixel(x, y);
  }
  void draw_box(short x1, short y1, short x2, short y2, char c)
  {
    _setcolor(c);
    _rectangle(_GBORDER, x1, y1, x2, y2);
  }
  // note:  this expects 25 x 40 coords (x & y swapped) and an 8x8 font
  void put_text(short x, short y, char c, char *text)
  {
    _settextcolor(c);
    _settextposition(x, y);
    _outtext(text);
  }
/*  #define BLACK 0
  #define LGRAY 8
  #define DGRAY 7
  #define WHITE 15*/
  #define BLACK 28
  #define LGRAY 24
  #define DGRAY 20
  #define WHITE 16
#endif

//======== I/O FUNCTIONS ==================================================
void wait_for_key()
{
  while (!kbhit());
}

void pause()
{
  wait_for_key();
  while (!getch());   // need to be called twice for extended keys
}

short get_num(char *prompt)
{
  char str[16];
    
  printf(prompt);
  fgets(str, 16, stdin);
  if (strlen(str) == 1)
    return -1;
  return(atoi(str) % 256);
}

void get_numd(char *prompt, char *num)
{
  char str[16];
    
  printf(prompt);
  fgets(str, 16, stdin);
  if (strlen(str) == 1)
    return;
  *num = atoi(str) % 256;
}

//======== SCREEN FUNCTIONS ===============================================
void new_screen()
{
  char x;
  
  x = map.num_maps;
  map.num_maps++;
  clear_screen(x);  
  memset(screen, 0, 128);
}

void clear_screen(char num)
{
  memset(map.screens[num], 0, 128);
  memset(map.info[num].edges, 0, 4);
  map.info[num].num_warps = 0;
  map.info[num].num_people = 0;
  map.info[num].num_enemies = 0;
}

void draw_screen()
{
  short x, y;
  
  for(y = 0; y < 8; y++)
    for(x = 0; x < 16; x++)
        draw_sprite((x * 8) + 97, (y * 8) + 1, screen[x][y]);
}

void view_screen(short i)
{
  short x;
  
  printf("\n");
  printf("---------------[Map %i]--------------\n", i);
  printf("Edges:\t\t    up=%3i\t  down=%3i\t  left=%3i\t right=%3i\n", map.info[i].edges[0], map.info[i].edges[1], map.info[i].edges[2], map.info[i].edges[3]);
  printf("Warps:   %3i", map.info[i].num_warps);
  if (map.info[i].num_warps == 0)
    printf("\n");
  for(x = 0; x < map.info[i].num_warps; x++)
  {
    if (x != 0)
      printf("\t");
    printf("\t   map=%3i\tsprite=%3i\t new y=%3i\t new x=%3i\n", map.info[i].warps[x][0], map.info[i].warps[x][1], map.info[i].warps[x][2], map.info[i].warps[x][3]);
  }
  printf("People:  %3i", map.info[i].num_people);
  if (map.info[i].num_people == 0)
    printf("\n");
  for(x = 0; x < map.info[i].num_people; x++)
  {
    if (x != 0)
      printf("\t");
    printf("\tsprite=%3i\t  text=%3i\n", map.info[i].people[x][0], map.info[i].people[x][1]);
  }
  printf("Enemies: %3i", map.info[i].num_enemies);
  if (map.info[i].num_enemies == 0)
    printf("\n");
  for(x = 0; x < map.info[i].num_enemies; x++)
  {
    if (x != 0)
      printf("\t");
    printf("\t enemy=%3i\t     y=%3i\t     x=%3i\n", map.info[i].enemies[x][0], map.info[i].enemies[x][1], map.info[i].enemies[x][2]);
  }
  printf("\n");
}

void edit_screen()
{
  short x, y,     // sprite list cursor
        a, b;     // map cursor
  char  text[20];
  
  set_gfx_mode();
  draw_sprites();
  put_text(12, 15, 8, "ZEdit v0.15");
  draw_screen();

  draw_box(96, 0, 225, 65, 1);    // map box
  draw_box(1, 101, 10, 110, 2);   // map sprite cursor
  draw_box(97, 1, 104, 8, 2);     // edit sprite cursor
  draw_box(39, 23, 56, 40, 1);    // current edit sprite view box
  draw_box(263, 23, 280, 40, 1);  // current map sprite view box
        
  put_text(7, 6, 2, " 0");
  sprintf(text, "%3i", screen[a][b]);
  put_text(7, 33, 2, text);
  put_text(10, 14, 2, "(  0,  0) = 0");

  x = 0;
  y = 0;
  a = 0;
  b = 0;
  for(;;)
  {
    wait_for_key();
    switch (getch())
    {
      case 27:        // escape
        set_text_mode();
        return;
      case 13:        // enter
      case 32:        // space
        screen[a][b] = (y * 29) + x;
        draw_sprite((a * 8) + 97, (b * 8) + 1, (y * 29) + x);
        draw_sprite_big(264, 24, screen[a][b]);
        sprintf(text, "%3i", screen[a][b]);
        put_text(7, 33, 2, text);
        draw_box((a * 8) + 97, (b * 8) + 1, 
                  (a * 8) + 104, (b * 8) + 8, 2);
        break;
      case 0:
        draw_box((x * 11) + 1, (y * 11) + 101,
                  (x * 11) + 10, (y * 11) + 110, 0);
        draw_box((a * 8) + 97, (b * 8) + 1, 
                  (a * 8) + 104, (b * 8) + 8, 0);
        draw_sprite((a * 8) + 97, (b * 8) + 1, screen[a][b]);
        switch (getch())
        {
          case 72:    // up
            y--;
            if (y < 0)
            {
              if (x > 23)
                y = 7;
              else
                  y = 8;
            }
            break;
          case 80:    // down
            y++;
            if (((y > 7) && (x > 23)) || (y > 8))
              y = 0;
            break;
          case 75:    // left
            x--;
            if (x < 0)
              if (y < 8)
                x = 28;
              else
                  x = 23;
            break;
          case 77:    // right
            x++;
            if ((x > 28) || ((x > 23) && (y == 8)))
              x = 0;
            break;
          case 71:    // home
            if (b > 0)
              b--;
            break;
          case 79:    // end
            if (b < 7)
              b++;
            break;
          case 83:    // delete
            if (a > 0)
              a--;
            break;
          case 81:    // page down
            if (a < 15)
              a++;
        }
        draw_sprite_big(40, 24, (y * 29) + x);
        sprintf(text, "%3i", (y * 29) + x);
        put_text(7, 5, 2, text);
        draw_sprite_big(264, 24, screen[a][b]);
        sprintf(text, "%3i", screen[a][b]);
        put_text(7, 33, 2, text);
        draw_box((x * 11) + 1, (y * 11) + 101,
                  (x * 11) + 10, (y * 11) + 110, 2);
        draw_box((a * 8) + 97, (b * 8) + 1, 
                  (a * 8) + 104, (b * 8) + 8, 2);
        sprintf(text, "(%3i, %2i) = %i  ", a * 8, b * 8, (b * 16) + a);
        put_text(10, 14, 2, text);
    }
  }
}

void change_screen(short n)
{
  short i, x;
  char  z;
  
  for(;;)
  {
    view_screen(n);
    printf("1) Edges\n");
    printf("2) Warps\n");
    printf("3) People\n");
    printf("4) Enemies\n");
    printf("5) Main Menu\n");
    printf("\n");
    i = get_num("] ");
    printf("\n");
    switch (i)
    {
      case 1:
        get_numd("   up: ", &map.info[n].edges[0]);
        get_numd(" down: ", &map.info[n].edges[1]);
        get_numd(" left: ", &map.info[n].edges[2]);
        get_numd("right: ", &map.info[n].edges[3]);
        break;
      case 2:
        z = 5;
        get_numd("warps: ", &z);
        if (z > 4)
          break;
        i = z;
        map.info[n].num_warps = i;
        for(x = 0; x < i; x++)
        {
          get_numd("   map: ", &map.info[n].warps[x][0]);
          get_numd("sprite: ", &map.info[n].warps[x][1]);
          get_numd(" new y: ", &map.info[n].warps[x][2]);
          get_numd(" new x: ", &map.info[n].warps[x][3]);
        }
        break;
      case 3:
        z = 5;
        get_numd("people: ", &z);
        if (z > 4)
          break;
        i = z;
        map.info[n].num_people = i;
        for(x = 0; x < i; x++)
        {
          get_numd("sprite: ", &map.info[n].people[x][0]);
          get_numd("  text: ", &map.info[n].people[x][1]);
        }
        break;
      case 4:
        z = 11;
        get_numd("enemies: ", &z);
        if (z > 10)
          break;
        i = z;
        map.info[n].num_enemies = i;
        for(x = 0; x < i; x++)
        {
          get_numd("enemy: ", &map.info[n].enemies[x][0]);
          get_numd("    y: ", &map.info[n].enemies[x][1]);
          get_numd("    x: ", &map.info[n].enemies[x][2]);
        }
        break;
      case 5:
        return;
    }
  }
}

void edit_chests()
{
  char x, y;

  for(;;)
  {
    printf("Chests:\n");
    printf("\n");
    for(x = 0; x < 20; x++)
      for(y = 0; y < 100; y += 20)
        printf("%2i) %3i %3i %2i  ", x + y, map.chests[x + y][0], map.chests[x + y][1], map.chests[x + y][2]);
    printf("\n");
    x = 100;  
    get_numd("Chest number: ", &x);
    if (x == 100)
    {
      printf("\n");
      return;
    }
    get_numd("   map: ", &map.chests[x][0]);
    get_numd("offset: ", &map.chests[x][1]);
    get_numd("  item: ", &map.chests[x][2]);
    printf("\n");
  }
}

//======== MAP FUNCTIONS ==================================================
void new_map()
{
  short x;
  
  memset(map.chests, 0, 300);
  for(x = 0; x < 100; x++)
    map.chests[x][0] = 0xff;
  map.num_maps = 0;
  new_screen();
}

void load_map(char *name)
{
  short len,
        p, m, x;

  len = read_string(name);
  if (len == 0)
  {
    new_map();
    return;
  }
  
  map.num_maps = str_read[0];
  p = 3 + (map.num_maps * 2);
  for(x = 0; x < map.num_maps; x++)
  {
    memcpy(map.screens[x], &str_read[p], 128);
    p += 128;
  }
  memcpy(map.chests, &str_read[p], 300);
  p += 300;
  for(m = 0; m < map.num_maps; m++)
  {
    memcpy(map.info[m].edges, &str_read[p], 4);
    p += 4;
    map.info[m].num_warps = str_read[p] / 4;
    p++;
    for(x = 0; x < map.info[m].num_warps; x++)
    {
      memcpy(map.info[m].warps[x], &str_read[p], 4);
      p += 4;
    }
    map.info[m].num_people = str_read[p] / 2;
    p++;
    for(x = 0; x < map.info[m].num_people; x++)
    {
      memcpy(map.info[m].people[x], &str_read[p], 2);
      p += 2;
    }
    map.info[m].num_enemies = str_read[p];
    p++;
    for(x = 0; x < map.info[m].num_enemies; x++)
    {
      memcpy(map.info[m].enemies[x], &str_read[p], 3);
      p += 3;
    }
  }
}

void save_map(char *name)
{
  char  data[16384];
  short p, m, x,
        len,
        info[256];
  
  data[0] = map.num_maps;
  data[1] = (map.num_maps * 130) % 256;
  data[2] = (map.num_maps * 130) >> 8;
  p = 3 + (map.num_maps * 2);
  for(m = 0; m < map.num_maps; m++)
  {
    memcpy(&data[p], map.screens[m], 128);
    p += 128;
  }
  memcpy(&data[p], map.chests, 300);
  p += 300;
  for(m = 0; m < map.num_maps; m++)
  {
    info[m] = p;
    memcpy(&data[p], map.info[m].edges, 4);
    p += 4;
    data[p] = map.info[m].num_warps * 4;
    p++;
    for(x = 0; x < map.info[m].num_warps; x++)
    {
      memcpy(&data[p], map.info[m].warps[x], 4);
      p += 4;
    }
    data[p] = map.info[m].num_people * 2;
    p++;
    for(x = 0; x < map.info[m].num_people; x++)
    {
      memcpy(&data[p], map.info[m].people[x], 2);
      p += 2;
    }
    data[p] = map.info[m].num_enemies;
    p++;
    for(x = 0; x < map.info[m].num_enemies; x++)
    {
      memcpy(&data[p], map.info[m].enemies[x], 3);
      p += 3;
    }
  }
  len = p;
  p = 3;
  for(x = 0; x < map.num_maps; x++)
  {
    data[p++] = info[x] % 256;
    data[p++] = info[x] >> 8;
  }
  
  write_string(name, data, len);
}

void insert_sprite(short n)
{
  short m, x;

  for(m = 0; m < map.num_maps; m++)
    for(x = 0; x < 128; x++)
      if ((map.screens[m][x] >= n) && (map.screens[m][x] < 255))
        map.screens[m][x]++;
}

void move_sprite(short s, short d)
{
  short m, x;

  for(m = 0; m < map.num_maps; m++)
    for(x = 0; x < 128; x++)
      if (map.screens[m][x] == s)
        map.screens[m][x] = d;
}

void edit_map()
{
  short i, x, y, z;
  
  for(;;)
  {
    printf("\n");
    printf("+---------------------+\n");
    printf("|     ZEdit v0.15     |\n"); 
    printf("| Zelda 86 Map Editor |\n");
    printf("+---------------------+\n");
    printf("Maps: %i\n", map.num_maps);
    printf("\n");
    printf("1) Add map\n");
    printf("2) Clear map\n");
    printf("3) Edit map\n");
    printf("4) Copy map\n");
    printf("5) Change map info\n");
    printf("6) Edit chests\n");
    printf("7) Insert sprites\n");
    printf("8) Move sprite\n");
    printf("9) Quit\n");
    printf("\n");
    i = get_num("] ");
    printf("\n");
    switch (i)
    {
      case 1:
        new_screen();
        break;
      case 2:
        i = get_num("Map to clear: ");
        if (i == -1)
          break;
        clear_screen(i);
        break;
      case 3:
        i = get_num("Map to edit: ");
        if (i == -1)
          break;
        z = 0;
        for(y = 0; y < 8; y++)
          for(x = 0; x < 16; x++)
            screen[x][y] = map.screens[i][z++];
        edit_screen();
        z = 0;
        for(y = 0; y < 8; y++)
          for(x = 0; x < 16; x++)
            map.screens[i][z++] = screen[x][y];
        break;
      case 4:    
        i = get_num("Source map: ");
        if (i == -1)
          break;
        z = get_num("Target map: ");
        if (z == -1)
          break;
        memcpy(map.screens[z], map.screens[i], 128);
        break;
      case 5:
        i = get_num("Map to change: ");
        if (i == -1)
          break;
        change_screen(i);
        break;
      case 6:
        edit_chests();
        break;
      case 7:
        i = get_num("Sprites to insert: ");
        if (i == -1)
          break;
        z = get_num("Starting number: ");
        if (z == -1)
          break;
        for(x = 0; x < i; x++)
          insert_sprite(z);
        break;
      case 8:
        i = get_num("Sprite to move: ");
        if (i == -1)
          break;
        z = get_num("Target sprite: ");
        if (z == -1)
          break;
        move_sprite(i, z);
        break;
      case 9:
        return;
    }
  }
}

void dump_maps_bmp(char *fname)
{
  char BMP_HEADER[] = {
    0x42,0x4d,0x36,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0x04,0x00,0x00,
    0x28,0x00,0x00,0x00,0x82,0x00,0x00,0x00,0x42,0x00,0x00,0x00,0x01,0x00,
    0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0xc4,0x0e,0x00,0x00,
    0xc4,0x0e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x80,0x00,0x00,0x00,0x80,0x80,0x00,
    0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0xc0,0xc0,
    0xc0,0x00,0x80,0x80,0x80,0x00,0x00,0x00,0xff,0x00,0x00,0xff,0x00,0x00,
    0x00,0xff,0xff,0x00,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xff,
    0x00,0x00,0xff,0xff,0xff,0x00,0x08,0x09,0x07,0x00,0x3e,0x0e,0x87,0x00,
    0x19,0x1c,0x7f,0x00,0x78,0xbc,0x00,0x00,0x00,0x00,0xe4,0x00,0x56,0x00,
    0x6d,0x00,0xf7,0xbf,0xbf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x00,
    0xf5,0xbf,0x0f,0x00,0xec,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x58,0x00,0x50,0x19,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x09,0x07,0x00,0x6c,0x74,
    0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0xff,0xff,0xff,0x00,0x00,0x00,0x01,0x00,0x8f,0x16,0x7e,0x00,0x8b,0x80,
    0x87,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x08,0x09,0x07,0x00,0x6c,0x74,0x02,0x00,0x00,0x00,
    0x02,0x00,0x1c,0xbc,0x08,0x00,0xa4,0x84,0x12,0x00,0x6c,0x74,0x02,0x00,
    0x00,0x00,0x37,0x00,0x67,0x01,0x07,0x00,0x00,0x00,0x67,0x00,0x00,0x00,
    0x78,0x00,0x02,0x00,0x07,0x00,0x08,0x09,0x00,0x00,0x37,0x43,0x1c,0x00,
    0x02,0x00,0xb0,0x00,0x74,0x04,0x87,0x00,0x6c,0x74,0x02,0x00,0x79,0x04,
    0x1a,0x00,0xf7,0xbf,0xac,0x00,0x56,0x00,0x10,0x00,0x56,0x00,0xd3,0x00,
    0xf7,0xbf,0x96,0x00,0x4d,0x5f,0x04,0x00,0x56,0x00,0xac,0x00,0x56,0x00,
    0xac,0x00,0x56,0x00,0xec,0x00,0x56,0x00,0xf2,0x00,0x49,0x5f,0x00,0x00,
    0x00,0x00,0xf8,0x00,0x56,0x00,0x54,0x00,0x40,0x5f,0xac,0x00,0x56,0x00,
    0xac,0x00,0x56,0x00,0x08,0x00,0x56,0x00,0xc4,0x00,0x49,0x5f,0xff,0x00,
    0xff,0xff,0x14,0x00,0x56,0x00,0xfb,0x00,0x41,0x5f,0x01,0x00,0x00,0x00,
    0xac,0x00,0x56,0x00,0xe4,0x00,0x56,0x00,0xd4,0x00,0x49,0x5f,0xff,0x00,
    0xff,0xff,0xf0,0x00,0x56,0x00,0xb8,0x00,0x01,0x01,0x38,0x00,0x03,0x01,
    0xb0,0x00,0x69,0x00,0x00,0x00,0x00,0x00,0x43,0x00,0x5c,0x4d,0x79,0x00,
    0x44,0x6f,0x63,0x00,0x6d,0x65,0x6e,0x00,0x73,0x00,0x00,0x00,0x00,0x00,
    0x87,0x00,0xe3,0x10,0x17,0x00,0x13,0x00,0x1e,0x00,0x7f,0x00,0x17,0x00,
    0x13,0x00,0x00,0x00,0x00,0x02,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x37,0x00,0x8c,0x0d,0x02,0x00,0x00,0x00,0x37,0x00,0xe8,0x07,0x88,0x00,
    0x7c,0x0b,0x00,0x00,0x02,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x02,
    0x00,0x00,0x00,0x00,0xa8,0x00,0x37,0x29,0x37,0x00,0x00,0x00,0x2c,0x00,
    0x8f,0x16,0x00,0x00,0x37,0x43,0xd4,0x00,0x7e,0x9e,0x67,0x00,0x00,0x00,
    0x01,0x00,0x02,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
    0xa8,0x85,0x37,0x00,0x1c,0xbc,0x5c,0x00,0x69,0x00,0xe8,0x00,0x00,0x02,
    0x00,0x00,0x17,0x00,0x13,0x00,0xb0,0xe5,0xed,0x00,0x1e,0x01,0x7f,0x00,
    0x00,0x00,0x8c,0x00,0x02,0x00,0x2c,0x00,0x5c,0x1e,0xce,0x00,0x8d,0xa9,
    0x01,0x00,0x2c,0x1e,0x5c,0x00,0xdc,0x85,0xdc,0x00,0x3c,0xaa,0x00,0x00,
    0x1c,0xbc,0x00,0x00,0x37,0x43,0xca,0x00,0x7e,0x49,0x7f,0x00,0x01,0x00,
    0x12,0x00,0x00,0x00,0x0c,0x00,0x69,0x00,0x5c,0x00,0xaf,0x16,0xd9,0x00,
    0xf7,0xbf,0x5f,0x00,0x40,0x5f,0x00,0x00,0x56,0x00,0x67,0x00,0xf5,0xbf,
    0x47,0x00,0xf7,0xbf,0x70,0x00,0x69,0x00,0x70,0x00,0x69,0x00,0x72,0x00,
    0x40,0x5f,0x34,0x00,0x56,0x00,0xf2,0x00,0x40,0x5f,0x01,0x00,0x00,0x00,
    0xa4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x58,0x00,
    0x00,0x00,0x57,0x00,0x72,0x64,0x50,0x00,0x72,0x66,0x65,0x00,0x74,0x20,
    0x47,0x00,0x61,0x70,0x68,0x00,0x63,0x73,0x00,0x00,0x69,0x63,0x73,0x00,
    0x61,0x74,0x00,0x00,0x69,0x00,0x8b,0x00,0x40,0x5f,0x00,0x00,0x00,0x02,
    0x2c,0x00,0x40,0x5f,0xb0,0x00,0x69,0x00,0x00,0x00,0x00,0x00,0x01,0x00,
    0x00,0x00,0x70,0x00,0x69,0x00,0x70,0x00,0x69,0x00,0x60,0x00,0x50,0x00,
    0x0a,0x00,0x56,0x00,0xd6,0x00,0x56,0x00,0xde,0x00,0x56,0x00,0xbc,0x00,
    0x56,0x00,0x17,0x00,0xf7,0xbf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0xde,0x00,0x56,0x00,0x37,0x00,0xda,0x86,0x37,0x00,0x4d,0x41,0xaf,0x00,
    0x00,0x00,0x00,0x00,0x58,0x0e,0x00,0x00,0x20,0x00,0x00,0x00,0xa4,0x02,
    0x00,0x00,0xa4,0x02,0x24,0x00,0x49,0x5f,0x01,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x00,0xdc,0x87,
    0xbf,0x00,0xdf,0x05,0x57,0x00,0x00,0x00,0x17,0x00,0x7a,0xfc,0x67,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0x00,0x26,0x80,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0x60,0x00,0x00,
    0xe0,0x61,0x01,0x00,0x1c,0x87,0xdf,0x00,0x3f,0x01,0xe2,0x00,0xe0,0x61,
    0x01,0x00,0x57,0x01,0x5e,0x00,0xe9,0x4a,0x3f,0x00,0x64,0x13,0x57,0x00,
    0x78,0x60,0x00,0x00,0xe0,0x61,0x01,0x00,0x5e,0x87,0x56,0x00,0x46,0x87,
    0x56,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x22,0x00,0x01,0x00,0x00,0x00,
    0xe0,0x61,0x01,0x00,0x6f,0x03,0x57,0x00,0xcf,0x00,0x00,0x00,0xc0,0x54,
    0x5e,0x00,0x5c,0x87,0x00,0x00,0x0c,0x00,0x01,0x00,0xe4,0x61,0xcf,0x00,
    0x58,0x4e,0x8f,0x00,0x40,0x00,0x00,0x00,0x6f,0x03,0xef,0x00,0x00,0x00,
    0x90,0x00,0x06,0x00,0xee,0x00,0x36,0x0a,0x5a,0x00,0x62,0xb2,0x00,0x00,
    0x00,0x00,0x92,0x00,0x3a,0x64,0x36,0x00,0x26,0x02,0xd2,0x00,0x4d,0x4f,
    0x7f,0x00,0x01,0x00,0x16,0x00,0x00,0x00,0x17,0x00,0xa8,0x87,0x48,0x00,
    0xa7,0x04,0x36,0x00,0x00,0x00,0x8e,0x00,0xca,0x8b,0x8e,0x00,0x62,0xb2,
    0x62,0x00,0x17,0x06,0xe4,0x00,0x99,0x13,0xb7,0x00,0x3e,0x14,0xba,0x00 };
  
  FILE  *fp;
  short x;  

  fp = fopen(fname, "wb");
  if (fp == NULL)
  {
    printf("\nError opening %s.  Bitmap not saved!\n", fname);
    return;
  }
  BMP_HEADER[22] = (map.num_maps * 66) % 256;
  BMP_HEADER[23] = (map.num_maps * 66) / 256;
  fwrite(BMP_HEADER, 1078, 1, fp);
  for(x = map.num_maps - 1; x >= 0; x--)
    write_map_bmp(fp, x);
  fclose(fp);
  if (errno != 0)
  {
    printf("\nError opening %s.  Bitmap not saved!\n", fname);
    return;
  }
  printf("Bitmap saved as %s\n", fname);
}

void write_map_bmp(FILE *fp, short i)
{
  char  *bmp;
  short a, b, x, y;
  char  s, m, p;
  
  bmp = malloc(8712);
  memset(bmp, 12, 8712);  // blue

  for(b = 0; b < 8; b++)
    for(a = 0; a < 16; a++)
    {
      s = map.screens[i][(b * 16) + a];
      for(y = 0; y < 8; y++)
      {
        m = 0x80;
        for(x = 0; x < 8; x++) 
        {
          if ((sprites[s][y] & m) / m == 1)
          {
            if ((sprites[s][y + 8] & m) / m == 1)
              p = 0;    // white
            else
                p = 8;  // dark gray
          }
          else
          {
            if ((sprites[s][y + 8] & m) / m == 1)
              p = 7;     // light gray
            else
                p = 15;  // black
          }
          bmp[((7 - b) * 1056) + (a * 8) + ((7 - y) * 132) + x + 133] = p;
          m >>= 1;
        }
      }
    }

  fwrite(bmp, 8712, 1, fp);
  free(bmp);
}

//======== SPRITE FUNCTIONS ===============================================
void read_sprites(char *fname)
{
  FILE        *fp;
  FileHeader  header;
  FileEntry   entry;
  VarString   str;

  fp = fopen(fname, "rb");
  if (fp == NULL)
  {
    printf("Cannon open file %s -- make sure it is in\n", fname);
    printf("the same directory (folder) that you ran ZEdit from.\n");
    exit(1);
  }
  else
  {
    fread(&header, sizeof(FileHeader), 1, fp);
    fread(&entry, sizeof(FileEntry), 1, fp);
    fread(&str, sizeof(VarString), 1, fp);
      
    fread(&numspr, 1, 1, fp);
    fread(&sprites, 16, numspr, fp);

    fclose(fp);
    if (errno != 0)
    {
      printf("Error reading %s -- file is corrupt!\n", fname);
      exit(1);
    }
  }
}

void draw_sprite(short x, short y, char spr)
{
  short row,
        col,
        color;
  char  bitmask;

  for(row = y; row < y + 8; row++)
  {
    bitmask = 0x80;
    for(col = x; col < x + 8; col++)
    {
      if ((sprites[spr][row - y] & bitmask) / bitmask == 1)
      {
        if ((sprites[spr][(row - y) + 8] & bitmask) / bitmask == 1)
          color = WHITE;
        else
            color = DGRAY;
      }
      else
      {
          if ((sprites[spr][(row - y) + 8] & bitmask) / bitmask == 1)
            color = LGRAY;
          else
              color = BLACK;
      }
      put_pixel(col, row, color);
      bitmask >>= 1;
    }
  }
}

void draw_sprite_big(short x, short y, char spr)
{
  short row,
        col,
        color;
  char  bitmask;

  for(row = y; row < y + 16; row += 2)
  {
    bitmask = 0x80;
    for(col = x; col < x + 16; col += 2)
    {
      if ((sprites[spr][(row - y) / 2] & bitmask) / bitmask == 1)
      {
        if ((sprites[spr][((row - y) / 2) + 8] & bitmask) / bitmask == 1)
          color = WHITE;
        else
            color = DGRAY;
      }
      else
      {
          if ((sprites[spr][((row - y) / 2) + 8] & bitmask) / bitmask == 1)
            color = LGRAY;
          else
              color = BLACK;
      }
      put_pixel(col, row, color);
      put_pixel(col + 1, row, color);
      put_pixel(col, row + 1, color);
      put_pixel(col + 1, row + 1, color);
      bitmask >>= 1;
    }
  }
}

void draw_sprites()
{
  short n, x, y;

  n = 0;
  x = 2;
  y = 102;
  while (n < numspr)
  {
    if (x > 310)
    {
      x = 2;
      y += 11;
    }
    draw_sprite(x, y, n);
    n++;
    x += 11;
  }
  draw_box(0, 100, 319, 199, 1);
}

//======== MAIN() =========================================================
void main(int argc, const char *argv[])
{
  read_sprites("zeldadat.86s");
  
  load_map("maps");
  edit_map();
  save_map("maps");
  dump_maps_bmp("maps.bmp");
}
